import { Telemetry, collect } from '@vuedx/shared';
import { createTypescriptLanguageService } from '@vuedx/vue-languageservice';
import * as Path from 'path';
import { performance } from 'perf_hooks';

var version = "0.7.4";

const PATCHED_METHODS = Symbol('Vue Patched Methods');
function overrideMethod(target, methodName, createOverride) {
    const patched = target[PATCHED_METHODS] ?? [];
    if (patched.includes(methodName)) {
        return () => {
            // @ts-expect-error
            if (target[methodName]?._original != null) {
                // @ts-expect-error
                target[methodName] = target[methodName]._original;
            }
        };
    }
    let fn = target[methodName];
    if (typeof fn === 'function') {
        try {
            fn = fn.bind(target);
        }
        catch {
            // - ignore
        }
    }
    target[methodName] = createOverride(fn);
    // @ts-expect-error
    target[methodName]._original = fn;
    patched.push(methodName);
    target[PATCHED_METHODS] = patched;
    return () => {
        target[methodName] = fn;
    };
}

const byProject = new Map();
let logger;
function isLanguageService(service) {
    return '_isVueTS' in service && service['_isVueTS'] === true;
}
function init({ typescript }) {
    Telemetry.setup('https://a1461052e1d94c7a9ee7c3f7add71b24@o237831.ingest.sentry.io/5595721', 'typescript-plugin-vue', version, 0.001, { typescriptVersion: typescript.versionMajorMinor });
    overrideMethod(typescript, 'getSupportedExtensions', (fn) => (options, extraFileExtensions) => {
        const extensions = fn(options, extraFileExtensions);
        const index = extensions.indexOf('.vue');
        if (index >= 0) {
            // File extensions are sorted in order of their priorities.
            // We need to put .vue before .ts
            extensions.splice(index, 1);
            extensions.unshift('.vue');
        }
        return extensions;
    });
    return {
        create(info) {
            let id;
            const start = performance.now();
            if (logger == null) {
                console.info = console.debug = console.log = (...args) => {
                    if (typeof args[0] === 'string') {
                        info.project.projectService.logger.info(args[0]);
                    }
                };
            }
            logger = info.project.projectService.logger;
            logger.info(`[VueDX] (plugin) Project: ${info.project.getProjectName()}. Other active projects: ${JSON.stringify(Array.from(byProject.keys()).map((project) => project.getProjectName()), null, 2)}`);
            if (isLanguageService(info.languageService)) {
                id = info.languageService._vueTS_id;
                logger.info(`[VueDX] (plugin) ${id} LanguageService is already enhanced.`);
                patchExtraFileExtensions(id, info.project);
                return info.languageService;
            }
            const previousProvider = byProject.get(info.project);
            if (previousProvider != null &&
                isLanguageService(previousProvider?.service)) {
                id = previousProvider.service._vueTS_id;
                logger.info(`[VueDX] (plugin) ${id} Found LanguageService provider.`);
                patchExtraFileExtensions(id, info.project);
                return previousProvider.service;
            }
            logger.info(`[VueDX] (plugin) Creating LanguageService provider.`);
            const onBeforeExit = () => {
                logger.info(`[VueDX] (plugin) ${id} Process is exiting.`);
            };
            process.on('beforeExit', onBeforeExit);
            const provider = createTypescriptLanguageService({
                ...info,
                typescript,
                typesDir: Path.resolve(__dirname, '..', 'runtime'),
                dispose() {
                    logger.info(`[VueDX] (plugin) ${id} Disposing LanguageService.`);
                    byProject.delete(info.project);
                    process.off('beforeExit', onBeforeExit);
                },
            });
            id = provider.service._vueTS_id;
            byProject.set(info.project, provider);
            const end = performance.now();
            collect('plugin_create', {
                startupTime: end - start,
            });
            logger.info(`[VueDX] (plugin) ${id} Plugin started in ${end - start}ms`);
            patchExtraFileExtensions(id, info.project);
            return provider.service;
        },
        getExternalFiles(project) {
            const service = byProject.get(project)?.service;
            if (service != null) {
                return service.getExternalFiles(project);
            }
            else {
                logger.info(`[VueDX] (plugin) Cannot find LanguageService provider for ${project.getProjectName()}`);
                return [];
            }
        },
        onConfigurationChanged(config) {
            logger.info(`[VueDX] (plugin) Update config for ${JSON.stringify(Array.from(byProject.keys()).map((project) => project.getProjectName()), null, 2)}`);
            byProject.forEach((provider) => {
                logger.info(`[VueDX] (plugin) ${provider.service._vueTS_id} Update config: ${JSON.stringify(config, null, 2)}`);
                provider.setConfig(config);
            });
        },
    };
}
function patchExtraFileExtensions(id, project) {
    const projectService = project.projectService;
    const extraFileExtensions = [
        {
            extension: '.vue',
            isMixedContent: false,
            scriptKind: 7, // deferred extra extensions are included in root files search.
        },
    ];
    const prefix = `[VueDX] (plugin) ${id} `;
    overrideMethod(projectService, 'setHostConfiguration', (setHostConfiguration) => {
        return (args) => {
            logger.info(prefix + 'setHostConfiguration: ' + JSON.stringify(args));
            const current = projectService
                .hostConfiguration.extraFileExtensions;
            logger.info(prefix + 'Current Extra Extensions: ' + JSON.stringify(current));
            if (args.extraFileExtensions != null) {
                args.extraFileExtensions.push(...extraFileExtensions);
            }
            else if (current == null) ;
            else if (current.every((ext) => ext.extension !== '.vue')) {
                args.extraFileExtensions = [...current, ...extraFileExtensions];
            }
            logger.info(prefix +
                'New Extra Extensions: ' +
                JSON.stringify(args.extraFileExtensions));
            return setHostConfiguration(args);
        };
    });
    const current = projectService
        .hostConfiguration.extraFileExtensions;
    if (Array.isArray(current) &&
        current.some((ext) => ext.extension === '.vue')) ;
    else if (Array.isArray(current)) {
        // Enable .vue after enhancing the language server.
        logger.info(prefix + 'Add extra extensions now.');
        projectService.setHostConfiguration({});
        return true;
    }
    return false;
}

/* eslint-disable import/first */
console.log = console.debug = console.info = console.debug;

export default init;
//# sourceMappingURL=index.mjs.map
